/* $Id: etunloop.c,v 1.3 2001/10/12 15:19:40 hiddy Exp $ */
/*-
 * Copyright (c) 2001 Hideaki Imaizumi <hiddy@sfc.wide.ad.jp>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *        This product includes software developed by the NetBSD
 *        Foundation, Inc. and its contributors.
 * 4. Neither the name of The NetBSD Foundation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/param.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/sockio.h>
#include <net/if.h>
#include <net/if_dl.h>
#include <net/if_types.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/* ioctl's for get/set debug */
#define ETUNIOSDEBUG	_IOW('e', 100, int)
#define ETUNIOSHASFCS	_IOW('e', 101, int)
#define ETUNIOIFATTACH	_IOW('e', 102, struct ifreq *)
#define ETUNIOIFDETACH	_IOR('e', 103, int)

int ifup(int soc, char *ifname, int yes);

void usage_exit()
{
	fprintf(stderr, "Usage: etunloop [-d /dev/etun?] [-b ifname]\n");
	exit(1);
}

int 
main(int argc, char **argv)
{
	char tundev[128], *bridgeif;
	int tunfd;
	int nr, nw;
	int c,  n,  yes = 1, no = 0;
	char buf[1600];
	struct ifreq ifr;
	int soc;

	strcpy(tundev, "/dev/etun0");
	while((c = getopt(argc, argv, "d:b:")) != EOF){
		switch(c){
		case 'd':
			strcpy(tundev, optarg);
			break;
		case 'b':
			bridgeif = optarg;
			break;
		default:
			usage_exit();
		}
	}

	tunfd = open(tundev, O_RDWR);
	if(tunfd < 0){
		perror("open(tundev)");
		exit(1);
	}
	
	if(ioctl(tunfd, ETUNIOSDEBUG, &yes) < 0){
		perror("ioctl(ETUNIOSDEBUG)");
		exit(1);
	}

	if(bridgeif){
                memcpy(ifr.ifr_name, bridgeif, IFNAMSIZ);
                if(ioctl(tunfd, ETUNIOIFATTACH, &ifr) < 0){
                        perror("ioctl(ETUNIOIFATTACH)");
			exit(1);
                }
                printf("tunbridge: attach %s\n", bridgeif);
        }


	if(ioctl(tunfd, ETUNIOSHASFCS, &no) < 0){
		perror("ioctl(ETUNIOSHASFCS)");
		exit(1);
	}

	soc = socket(AF_INET, SOCK_STREAM, 0);
	if(soc < 0){
		perror("socket");
		exit(1);
	}

	if(ifup(soc, &tundev[5], 1) < 0){
		perror("ifup(tunif)");
		exit(1);
	}
	if(bridgeif && ifup(soc, bridgeif, 1) < 0){
		perror("ifup(bridgeif)");
		exit(1);
	}

	while((nr = read(tunfd, buf, 1600)) > 0){
		printf("etunloop: read %d bytes from tunnel.\n", nr);
		sleep(1);
		nw = write(tunfd, buf, nr);
		printf("etunloop: write %d bytes to tunnel.\n", nw);
		if(nw != nr)
			break;
	} 
	if(nr < 0)
		perror("read");
	else if(nw < 0)
		perror("write");

	close(tunfd);
}


int ifup(int soc, char *ifname, int yes)
{
	int ret, flags = 0;
	struct ifreq ifr;

	memcpy(ifr.ifr_name, ifname, IFNAMSIZ);
	if((ret = ioctl(soc, SIOCGIFFLAGS, &ifr)) < 0){
		perror("ioctl(SIOCGIFFLAGS)");
		return  -1;
	}

	flags = ifr.ifr_flags;	
	if((flags & IFF_UP) == (yes? 0 : IFF_UP)){
		ifr.ifr_flags = (yes? IFF_UP: 0);
		if((ret = ioctl(soc, SIOCSIFFLAGS, &ifr)) < 0){
			perror("ioctl(SIOCSIFFLAGS)");
			return -1;
		}
	}
	return 0;
}

